import xarray as xr
from typing import List
from pathlib import Path
from datetime import datetime
import logging


class BadDataset(RuntimeError):
    pass


def open_nc(filename):
    return xr.open_dataset(filename, use_cftime=True)


def fix_abs_datetime(broken_time):
    if broken_time.attrs.get("units", None) == "day as %Y%m%d.%f":
        time_as_string = f"{broken_time:08.0f}"
        broken_time = datetime.strptime(time_as_string, "%Y%m%d").date()
    try:
        timestr = f"{broken_time:%Y-%m-%d}"
    except Exception as e:
        print(e, broken_time)
        raise
    return timestr


def get_start_end(files: List[Path]):
    try:
        first = open_nc(files[0])
        start = first.time[0]
    except AttributeError as e:
        logging.error(f"Cannot read start time from {files[0]}: {e}")
        return dict(date_start="", date_end="")
    date_start = fix_abs_datetime(start)
    try:
        last = open_nc(files[-1])
        end = last.time[-1]
        date_end = fix_abs_datetime(end)
    except IndexError:
        logging.error(f"{files[-1]} has a broken time axis")
        try:
            last = open_nc(files[-2])
            end = last.time[-1]
            date_end = fix_abs_datetime(end)
            del files[-1]
        except IndexError:
            raise BadDataset(f"Can't deduce times from {files}") from None

    return dict(date_start=date_start, date_end=date_end)
